<?php

namespace TrainingWheels\Resource;

class GitFilesResource extends Resource {

  protected $dir;
  protected $repo;
  protected $course_name;

  /**
   * Constructor.
   */
  public function __construct(\TrainingWheels\Environment\TrainingEnv $env, $title, $user_name, $course_name, $dir, $repo) {
    parent::__construct($env, $title, $user_name);
    $this->dir = $dir;
    $this->repo = $repo;
    $this->course_name = $course_name;
  }

  /**
   * Get the info on this resource.
   */
  public function get() {
    $info = array(
      'type' => 'gitfiles',
      'exists' => $this->exists(),
      'title' => $this->title,
    );
    if ($info['exists']) {
      $info['attribs']['branch'] = $this->currentBranch();
      $info['attribs']['changes'] = $this->localChanges();
      $info['attribs']['remote'] = $this->remote();
    }
    return $info;
  }

  /**
   * Get the current branch.
   */
  public function currentBranch() {
    return $this->env->gitBranchGet($this->dir);
  }

  /**
   * Get any local changes.
   */
  public function localChanges() {
    $changes = $this->env->gitLocalChanges($this->dir);
    return empty($changes) ? FALSE : explode("\n", $changes);
  }

  /**
   * Get the details of the remote.
   */
  public function remote() {
    $remote = $this->env->gitRemote($this->dir);
    $remote = str_replace("\t", ' ', $remote);
    return empty($remote) ? FALSE : explode("\n", $remote);
  }

  /**
   * Do the files exist in the environment?
   */
  public function exists() {
    return $this->env->dirExists($this->dir);
  }

  /**
   * Delete the files.
   */
  public function delete() {
  }

  /**
   * Create the git clone in the correct place.
   */
  public function create() {
    if (!$this->exists()) {
      $this->env->gitRepoClone($this->user_name, $this->repo, $this->dir);
    }
    else {
      throw new Exception("The directory $this->dir already exists.");
    }
  }

  /**
   * Sync to a target.
   */
  public function syncTo(GitFilesResource $target) {
    $this->env->fileSyncUserFolder($this->user_name, $target->user_name, "www/$this->course_name/");
  }
}
