<?php

/**
 * @file
 * Drush commands for the Training Wheels module. These mostly need to be run
 * as root (or sudo as root).
 */

/**
 * Implements hook_drush_command().
 */
function twcore_drush_command() {
  $items['user-create'] = array(
    'description' => 'Create user(s)',
    'aliases' => array('uc'),
    'arguments' => array(
      'courseid' => 'Course id',
      'names' => 'The user names to create.',
    ),
  );
  $items['user-delete'] = array(
    'description' => 'Delete user(s).',
    'aliases' => array('ud'),
    'arguments' => array(
      'courseid' => 'Course id',
      'names' => 'The user names to delete.',
    ),
  );
  $items['user-retrieve'] = array(
    'description' => 'Retrieve info on user(s).',
    'aliases' => array('ur'),
    'arguments' => array(
      'courseid' => 'Course id',
      'names' => 'The user names to retrieve.',
    ),
  );
  $items['resource-create'] = array(
    'description' => 'Create resources for users.',
    'aliases' => array('rc'),
    'arguments' => array(
      'courseid' => 'Course id',
      'names' => 'The user names.',
      'resources' => 'The resource names',
    ),
  );
  $items['resource-sync'] = array(
    'description' => 'Sync resources.',
    'aliases' => array('rs'),
    'arguments' => array(
      'courseid' => 'Course id',
      'source_user' => 'The source user to sync from',
      'target_users' => 'The target users to sync to',
      'resources' => 'The resource names',
    ),
  );
  $items['debug-conn'] = array(
    'description' => 'Debug connection.',
    'aliases' => array('dc'),
    'arguments' => array(
      'courseid' => 'Course id',
    ),
  );
  return $items;
}

/**
 * Command callback.
 *
 * Create user(s).
 *
 * drush uc 1 instructor,cilantro,basil,tumeric
 *
 */
function drush_twcore_user_create($courseid, $user_names) {
  try {
    $course = twcore_factory_course_create($courseid);
    $course->usersCreate(explode(',', $user_names));
  }
  catch (Exception $e) {
    drush_print($e->getMessage());
  }
}

/**
 * Command callback.
 *
 * Delete user(s).
 *
 * drush ud 1 cilantro,basil,tumeric
 */
function drush_twcore_user_delete($courseid, $user_names) {
  try {
    $course = twcore_factory_course_create($courseid);
    $course->usersDelete(explode(',', $user_names));
  }
  catch (Exception $e) {
    drush_print($e->getMessage());
  }
}

/**
 * Command callback.
 *
 * Retrieve user(s).
 *
 * drush ur 1 cilantro,basil,tumeric
 */
function drush_twcore_user_retrieve($courseid, $user_names) {
  try {
    $course = twcore_factory_course_create($courseid);
    $users = $course->usersGet(explode(',', $user_names));
    drush_print_r($users);
  }
  catch (Exception $e) {
    drush_print($e->getMessage());
  }
}

/**
 * Command callback.
 *
 * Create resources for a user.
 *
 * drush rc 1 instructor all
 *
 */
function drush_twcore_resource_create($courseid, $user_names, $resources) {
  try {
    $resources = $resources == 'all' ? '*' : $resources;
    $course = twcore_factory_course_create($courseid);
    if ($resources != '*') {
      $resources = explode(',', $resources);
    }
    $out = $course->usersResourcesCreate(explode(',', $user_names), $resources);
    drush_print_r($out);
  }
  catch (Exception $e) {
    drush_print($e->getMessage());
  }
}

/**
 * Command callback.
 *
 * Sync resources from one user to another.
 *
 * drush rs 1 instructor cilantro,basil,tumeric all
 * drush rs 1 instructor basil mysqldb
 */
function drush_twcore_resource_sync($courseid, $source_user, $target_users, $resources) {
  try {
    $resources = $resources == 'all' ? '*' : $resources;
    $course = twcore_factory_course_create($courseid);
    if ($resources != '*') {
      $resources = explode(',', $resources);
    }
    $out = $course->usersResourcesSync($source_user, explode(',', $target_users), $resources);
    drush_print_r($out);
  }
  catch (Exception $e) {
    drush_print($e->getMessage());
  }
}

/**
 * Command callback.
 *
 * Debugging.
 *
 */
function drush_twcore_debug_conn($courseid) {
  try {
    $course = twcore_factory_course_create($courseid);
    drush_print_r($out);
  }
  catch (Exception $e) {
    drush_print($e->getMessage());
  }
}
